"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressHooks = void 0;
const api_1 = require("@opentelemetry/api");
const utils_1 = require("../utils");
exports.ExpressHooks = {
    requestHook(span, { req, res }) {
        api_1.diag.debug('opentelemetry-instrumentation-express on requestHook()');
        const oldResEnd = res.end;
        const oldResSend = res.send;
        if (req.query)
            span.setAttribute('http.request.query', JSON.stringify(req.query));
        if (req.headers)
            span.setAttribute('http.request.headers', JSON.stringify(req.headers));
        let response;
        res.send = function (data) {
            response = data;
            res.send = oldResSend;
            // eslint-disable-next-line prefer-rest-params
            return oldResSend.apply(res, arguments);
        };
        res.end = function () {
            api_1.diag.debug('opentelemetry-instrumentation-express on end()');
            return utils_1.safeExecute(() => {
                // eslint-disable-next-line prefer-rest-params
                const origRes = oldResEnd.apply(res, arguments);
                if (res.getHeaders())
                    span.setAttribute('http.response.headers', JSON.stringify(res.getHeaders()));
                if (response)
                    span.setAttribute('http.response.body', JSON.stringify(response));
                if (req.body)
                    span.setAttribute('http.request.body', JSON.stringify(req.body));
                res.end = oldResEnd;
                return origRes;
            })();
        };
    },
    responseHook(span, response) {
        api_1.diag.debug('opentelemetry-instrumentation-express on responseHook()');
    },
};
//# sourceMappingURL=express.js.map