"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpHooks = exports.extractBodyFromWriteOrEndFunc = exports.isEmptyString = exports.extractBodyFromEmitSocketEvent = exports.isEncodingType = exports.isValidHttpRequestBody = void 0;
const shimmer = __importStar(require("shimmer"));
const api_1 = require("@opentelemetry/api");
const utils_1 = require("../utils");
const awsSpan_1 = require("../spans/awsSpan");
// eslint-disable-next-line @typescript-eslint/no-empty-function
const noop = () => { };
const isFunctionAlreadyWrapped = (fn) => fn && fn.__wrapped;
const hook = (module, funcName, options = {}, shimmerLib = shimmer) => {
    const { beforeHook = noop, afterHook = noop } = options;
    const safeBeforeHook = utils_1.safeExecute(beforeHook, `before hook of ${funcName} fail`);
    const safeAfterHook = utils_1.safeExecute(afterHook, `after hook of ${funcName} fail`);
    const extenderContext = {};
    try {
        const wrapper = (originalFn) => {
            if (isFunctionAlreadyWrapped(originalFn))
                return originalFn;
            return function (...args) {
                safeBeforeHook.call(this, args, extenderContext);
                const originalFnResult = originalFn.apply(this, args);
                safeAfterHook.call(this, args, originalFnResult, extenderContext);
                return originalFnResult;
            };
        };
        shimmerLib.wrap(module, funcName, wrapper);
    }
    catch (e) {
        console.warn(`Wrapping of function ${funcName} failed`, options);
    }
};
const MAX_SIZE = 4084;
const createEmitResponseOnEmitBeforeHookHandler = (requestData, response, span) => {
    let body = '';
    const maxPayloadSize = MAX_SIZE;
    return function (args) {
        const { headers, statusCode } = response;
        if (args[0] === 'data' && body.length < maxPayloadSize) {
            let chunk = args[1].toString();
            const allowedLengthToAdd = maxPayloadSize - body.length;
            //if we reached or close to limit get only substring of the part to reach the limit
            if (chunk.length > allowedLengthToAdd) {
                chunk = chunk.substr(0, allowedLengthToAdd);
            }
            body += chunk;
        }
        if (args[0] === 'end') {
            requestData.response.body = body;
            requestData.response.headers = headers;
            requestData.response.statusCode = statusCode;
            requestData.request.path = span.attributes['http.target'];
            requestData.request.host = span.attributes['http.host'];
            span.setAttribute('http.response.body', body);
            try {
                if (utils_1.isAwsService(requestData.request.host, requestData.response)) {
                    span.setAttributes(awsSpan_1.getAwsServiceData(requestData.request, requestData.response, span));
                    span.setAttribute('aws.region', span.attributes['http.host'].split('.')[1]);
                }
            }
            catch (e) {
                console.warn('Failed to parse aws service data', e);
                console.warn('getHttpSpan args', { requestData });
            }
        }
    };
};
const isValidHttpRequestBody = (reqBody) => !!(reqBody && (typeof reqBody === 'string' || reqBody instanceof Buffer));
exports.isValidHttpRequestBody = isValidHttpRequestBody;
const isEncodingType = (encodingType) => !!(encodingType &&
    typeof encodingType === 'string' &&
    ['ascii', 'utf8', 'utf16le', 'ucs2', 'base64', 'binary', 'hex'].includes(encodingType));
exports.isEncodingType = isEncodingType;
const extractBodyFromEmitSocketEvent = (socketEventArgs) => {
    return utils_1.safeExecute(() => {
        var _a, _b;
        if (socketEventArgs &&
            socketEventArgs._httpMessage &&
            socketEventArgs._httpMessage._hasBody) {
            const httpMessage = socketEventArgs._httpMessage;
            let lines = [];
            // eslint-disable-next-line no-prototype-builtins
            if (httpMessage.hasOwnProperty('outputData')) {
                lines = ((_a = httpMessage.outputData[0]) === null || _a === void 0 ? void 0 : _a.data.split('\n')) || [];
                // eslint-disable-next-line no-prototype-builtins
            }
            else if (httpMessage.hasOwnProperty('output')) {
                lines = ((_b = httpMessage.output[0]) === null || _b === void 0 ? void 0 : _b.split('\n')) || [];
            }
            if (lines.length > 0) {
                return lines[lines.length - 1];
            }
        }
    }, 'failed to extractBodyFromEmitSocketEvent', 'warn', '')();
};
exports.extractBodyFromEmitSocketEvent = extractBodyFromEmitSocketEvent;
const isEmptyString = (str) => !!(!str || (typeof str === 'string' && str.length === 0));
exports.isEmptyString = isEmptyString;
const extractBodyFromWriteOrEndFunc = (writeEventArgs) => {
    return utils_1.safeExecute(() => {
        if (exports.isValidHttpRequestBody(writeEventArgs[0])) {
            const encoding = exports.isEncodingType(writeEventArgs[1]) ? writeEventArgs[1] : 'utf8';
            return typeof writeEventArgs[0] === 'string'
                ? new Buffer(writeEventArgs[0]).toString(encoding)
                : writeEventArgs[0].toString();
        }
    })();
};
exports.extractBodyFromWriteOrEndFunc = extractBodyFromWriteOrEndFunc;
const createEmitResponseHandler = (requestData, span) => {
    return (response) => {
        const onHandler = createEmitResponseOnEmitBeforeHookHandler(requestData, response, span);
        hook(response, 'emit', {
            beforeHook: onHandler,
        });
    };
};
const httpRequestWriteBeforeHookWrapper = (requestData, span) => {
    return function (args) {
        if (exports.isEmptyString(requestData.request.body)) {
            const body = exports.extractBodyFromWriteOrEndFunc(args);
            requestData.request.body += body;
            span.setAttribute('http.request.body', requestData.request.body);
        }
    };
};
const httpRequestEmitBeforeHookWrapper = (requestData, span) => {
    const emitResponseHandler = createEmitResponseHandler(requestData, span);
    const oneTimerEmitResponseHandler = utils_1.runOneTimeWrapper(emitResponseHandler, {});
    return function (args) {
        if (args[0] === 'response') {
            oneTimerEmitResponseHandler(args[1]);
        }
        if (args[0] === 'socket') {
            if (exports.isEmptyString(requestData.request.body)) {
                const body = exports.extractBodyFromEmitSocketEvent(args[1]);
                requestData.request.body += body;
                span.setAttribute('http.request.body', requestData.request.body);
            }
        }
    };
};
function getRequestHeaders(request) {
    return request.headers || request.getHeaders();
}
exports.HttpHooks = {
    requestHook(span, request) {
        api_1.diag.debug('@opentelemetry/instrumentation-http on requestHook()');
        utils_1.safeExecute(() => {
            const requestData = {
                request: {
                    body: '',
                    headers: {},
                },
                response: {
                    body: '',
                    headers: {},
                },
            };
            const headers = getRequestHeaders(request);
            requestData.request.host = (span === null || span === void 0 ? void 0 : span.attributes) && span.attributes['http.host'];
            if (headers) {
                requestData.request.headers = headers;
                span.setAttribute('http.request.headers', JSON.stringify(headers));
            }
            const emitWrapper = httpRequestEmitBeforeHookWrapper(requestData, span);
            const writeWrapper = httpRequestWriteBeforeHookWrapper(requestData, span);
            const endWrapper = (requestData, span) => {
                return function (args) {
                    if (exports.isEmptyString(requestData.request.body)) {
                        const body = exports.extractBodyFromWriteOrEndFunc(args);
                        requestData.request.body += body;
                        span.setAttribute('http.request.body', requestData.request.body);
                    }
                };
            };
            hook(request, 'end', { beforeHook: endWrapper });
            hook(request, 'emit', { beforeHook: emitWrapper });
            hook(request, 'write', { beforeHook: writeWrapper });
        })();
    },
    responseHook(span, response) {
        api_1.diag.debug('@opentelemetry/instrumentation-http on responseHook()');
        if (response.headers) {
            span.setAttribute('http.response.headers', JSON.stringify(response.headers));
        }
    },
};
//# sourceMappingURL=http.js.map